# standard imports
import platform
import subprocess
import sys
import os
import json

# third-party imprts
import distro
from eyantra_autoeval.utils.common import is_docker, run_shell_command
from rich import print
from rich.console import Console
from rich.prompt import Confirm

cb_count = 0
seen_frames = set()
cl_secret_pw = "ajracl2324"
base_link_to_frames = ['FWR', 'FWL', 'RWR', 'RWL']


def tf_callback(msg):
    global cb_count, seen_frames
    for transform in msg.transforms:
        if transform.header.frame_id == 'ebot_base_link' and transform.child_frame_id in base_link_to_frames:
            if transform.child_frame_id not in seen_frames:
                seen_frames.add(transform.child_frame_id)
    cb_count += 1


def get_joint_states(console):
    try:
        source_output = subprocess.run("bash -c 'source /opt/ros/humble/setup.bash'", shell=True)
        if source_output.returncode == 0:
            import rclpy
            from tf2_msgs.msg import TFMessage
            from rclpy.duration import Duration
        elif source_output.returncode == 1:
            console.print(f'[red]-> Failed to source ros setup. Try first running :\n\n\t\'source /opt/ros/humble/setup.bash\'\n[/red]')
            return ""
    except Exception as e:
        console.print(f'[red]-> Failed to import \'rclpy\'. Try first running :\n\n\t\'source /opt/ros/humble/setup.bash\'\n[/red]')

    global cb_count
      
    rclpy.init(args=None)
    node = rclpy.create_node('tf_subscriber_once_node')
    subscription = node.create_subscription(TFMessage, '/tf', tf_callback, 10)

    try:
        timeout_duration = Duration(seconds=3)
        timeout_seconds = timeout_duration.nanoseconds / 1e9
        start_time = node.get_clock().now()
        while rclpy.ok() and cb_count < 20 and (node.get_clock().now() - start_time) < timeout_duration:
            rclpy.spin_once(node, timeout_sec=timeout_seconds)
        if cb_count == 0:
            return ""
    except Exception as e:
        console.print(f'[red]-> Error casued due to: {str(e)}[/red]')
        return ""

    node.destroy_node()
    rclpy.shutdown()

    return list(seen_frames)


def evaluate():

    result = {}
    console = Console()

    with console.status("[bold green]Gathering data...") as status:

        result["autoeval_version"] = "0.1.1"

        console.print(f"[green]\n##############################################\nGathering system information[/green]")
        distribution = {
            "machine": platform.machine(),
            "release": platform.release(),
            "system": platform.system(),
            "version": distro.version(best=True),
            "name": distro.name(),
        }
        result["distro"] = distribution
        result["virtualized"] = "hypervisor" in run_shell_command("cat /proc/cpuinfo")
        result["dockerized"] = is_docker()


    self_os_declaration = True
    if (result["distro"]["name"] != "Ubuntu"
        or "22.04" not in result["distro"]["version"]
        # or result["distro"]["machine"] != "x86_64"
        or result["virtualized"]
        or result["dockerized"]):
        console.print(
            f"""You seem to be using [bold][blue]{result["distro"]["name"]} {result["distro"]["version"]} ({result["distro"]["machine"]}) on {"virtualized" if result["virtualized"] else ""}{"dockerized" if result["dockerized"] else ""} environment[/bold][/blue]. We only support [bold][blue]Ubuntu 22.04 on (x86_64) on baremetal[/bold][/blue]. You may continue to use the existing setup, and you shall not be penalized for the same. However, we’ll officially only support Ubuntu 22.04 as the OS of choice, and we won’t be to help you out with any installation related questions or package version incompatibility issues."""
        )
        self_os_declaration = Confirm.ask("Do you accept to take ownership of your setup and not seek help from e-Yantra for the same?")
    result["self-os-declaration"] = self_os_declaration

    if self_os_declaration:
        result["generate"] = True
    else:
        result["generate"] = False
        console.print(f"[bold][blue]Sorry, without providing confirmation, we can't proceed :(")
        console.print(f"[bold][red]JSON file is not generated.")
        return result

    with console.status("[bold green]Gathering data...") as status:

        console.print(f"[green]Gathering information on python[/green]")
        result["python"] = {}
        result["python"][
            "version"
        ] = f"{sys.version_info.major}.{sys.version_info.minor}.{sys.version_info.micro}"

        console.print(f"[green]Gathering information on ROS distribution[/green]")
        result["ros"] = {}
        result["ros"]["version"] = os.getenv("ROS_DISTRO")
        
        console.print(f"[green]Gathering information on gazebo[/green]")
        result["gazebo"] = {}
        result["gazebo"]["version"] = (
            subprocess.run(
                "gazebo --version | grep 'Gazebo' | awk '{print $NF}'",
                capture_output=True,
                shell=True,
            )
            .stdout.decode()
            .strip()
        )

        console.print(f"[green]Gathering information on gazebo real time factor[/green]")
        try:
            source_output = subprocess.run("bash -c 'gz stats -p -d 1'", shell=True, capture_output=True)
            if source_output.returncode == 0:
                if source_output.stderr.decode() == "An instance of Gazebo is not running.\n":
                    console.print(f'[red]-> Gazebo instance is not running. Launch ebot warehouse first :([/red]')
                    result["gazebo"]["RTF"] = ""
                else:
                    lines = source_output.stdout.decode().split('\n')
                    rtf_values = [line.split(',')[0].strip() for line in lines[1:-1]]
                    rtf_values = [float(value) for value in rtf_values]
                    if len(rtf_values) == 0:
                        console.print(f'[red]-> Real Time Factor for gazebo not found. Check if gazebo is properly running :([/red]')
                        result["gazebo"]["RTF"] = ""
                    else:
                        result["gazebo"]["RTF"] = rtf_values[-1]
            elif source_output.returncode == 127 and source_output.stderr.decode() == "bash: line 1: gz: command not found\n":
                console.print(f'[red]-> Gazebo not found. Try installing gazebo first.[/red]')
                result["gazebo"]["RTF"] = ""
            else:
                console.print(f'[red]-> Failed to get real time factor from gazebo. Try again :([/red]')
                result["gazebo"]["RTF"] = ""
        except Exception as e:
            console.print(f'[red]-> Gazebo not found. Check if gazebo is properly installed :([/red]')
            result["gazebo"]["RTF"] = ""

        console.print(f"[green]Gathering information on robot joint states[/green]")
        result["gazebo"]["joint_states"] = get_joint_states(console)

        console.print(f"[bold][blue]Done!")

        # console.print(result)

    return result







import rclpy
import shutil
from rclpy.node import Node
from rclpy.serialization import serialize_message
import rosbag2_py
import tf2_ros
from rich.console import Console
from tf2_msgs.msg import TFMessage
from gazebo_msgs.msg import LinkStates
from std_msgs.msg import String
from sensor_msgs.msg import JointState
class SimpleBagRecorder(Node):

    def __init__(self):

        super().__init__('simple_bag_recorder')

        self.console = Console()
        self.tf_buffer = tf2_ros.buffer.Buffer()
        self.r = tf2_ros.TransformBroadcaster(self)
        self.listener = tf2_ros.TransformListener(self.tf_buffer, self)
        self.writer = rosbag2_py.SequentialWriter()

        try:
            shutil.rmtree("my_bag")
        except:
            pass

        storage_options = rosbag2_py._storage.StorageOptions(
            uri='my_bag',
            storage_id='sqlite3')
        converter_options = rosbag2_py._storage.ConverterOptions('', '')

        self.writer.open(storage_options, converter_options)

        tf_topic_info = rosbag2_py._storage.TopicMetadata(
            name='tf',
            type='tf2_msgs/msg/TFMessage',
            serialization_format='cdr')
        
        
        fruit_states_topic_info = rosbag2_py._storage.TopicMetadata(
            name = "/_fruit_info",
            type = "std_msgs/msg/String",
            serialization_format = "cdr"
        )
        
        self.writer.create_topic(tf_topic_info)
        self.writer.create_topic(fruit_states_topic_info)
        
        self.subscription = self.create_subscription(
            TFMessage,
            'tf',
            self.topic_callback,
            10)
        self.create_subscription(
            String,
            '/_fruit_info',
            self.fruit_states_callback,
            10
        )
        self.timer_count = 0
        self.tf_cb_status = False
        self.timer_value = 8
        self.first_cb_status = False

        self.timer = self.create_timer(1.0, self.timer_callback)
        
    def topic_callback(self, msg):
        if not self.first_cb_status:
            self.timer_count = 0
        try:
            self.writer.write(
                'tf',
                serialize_message(msg),
                self.get_clock().now().nanoseconds)
            self.tf_cb_status = True
            self.first_cb_status = True
        except:
            self.tf_cb_status = False

    
    def fruit_states_callback(self,data):
        self.writer.write(
            '/_fruit_info',
            serialize_message(data),
            self.get_clock().now().nanoseconds
        )
    

    def timer_callback(self):
        if not self.tf_cb_status:
            self.console.print(f'[red]-> TF not published yet :( Waiting for data...[/red]')
        else:
            self.console.print(f'[green]-> Collecting TF data...[/green]')

        if self.timer_count > self.timer_value:
            raise SystemExit

        self.timer_count = self.timer_count + 1

def evaluate():
    
    result = {}
    console = Console()

    rclpy.init()
    sbr = SimpleBagRecorder()

    try:
        rclpy.spin(sbr)
    except KeyboardInterrupt:
        console.print(f"[red]\n-> Keyboard Interrupt :( Exiting...\nPlease try again without any interrupt![/red]")
    except SystemExit:
        console.print(f'[green]-> Data collection completed! Thankyou for being patient ;)\nFollow instructions on portal mdbook to submit task 1B.[/green]')

    result["generate"] = False

    return result

if __name__ == "__main__":
    evaluate()



import rclpy
import shutil
from rclpy.node import Node
from rclpy.serialization import serialize_message
import rosbag2_py
import tf2_ros
from tf2_msgs.msg import TFMessage
from rich.console import Console
from nav_msgs.msg import Odometry
from geometry_msgs.msg import PoseStamped, Twist, TwistStamped
from ros_gz_interfaces.msg import Contacts
from std_msgs.msg import String
from rosgraph_msgs.msg import Clock
import json
import logging
from sensor_msgs.msg import JointState
from rclpy.qos import QoSProfile, QoSReliabilityPolicy, QoSHistoryPolicy
from ros2node.api import get_absolute_node_name, get_node_names, get_topics
from ros2topic.api import get_topic_names_and_types


class SimpleBagRecorder(Node):
    def __init__(self):
        super().__init__("simple_bag_recorder")
        self.tf_buffer = tf2_ros.buffer.Buffer()
        self.r = tf2_ros.TransformBroadcaster(self)
        self.listener = tf2_ros.TransformListener(self.tf_buffer, self)
        self.writer = rosbag2_py.SequentialWriter()
        try:
            shutil.rmtree("my_bag")
        except:
            pass
        storage_options = rosbag2_py._storage.StorageOptions(
            uri="my_bag", storage_id="sqlite3"
        )
        converter_options = rosbag2_py._storage.ConverterOptions("", "")
        self.writer.open(storage_options, converter_options)

        # topic_info_ros_info = rosbag2_py._storage.TopicMetadata(
        #     name='_ros_info',
        #     type= 'std_msgs/msg/String',
        #     serialization_format='cdr')

        self.topic_info_list = [
            rosbag2_py._storage.TopicMetadata(
                name="tf",
                type="tf2_msgs/msg/TFMessage",  #'std_msgs/msg/String',
                serialization_format="cdr",
            ),
            rosbag2_py._storage.TopicMetadata(
                name="odom",
                type="geometry_msgs/msg/PoseStamped",  #'std_msgs/msg/String',
                serialization_format="cdr",
            ),
            rosbag2_py._storage.TopicMetadata(
                name="detection_status",
                type="std_msgs/msg/String",  #'std_msgs/msg/String',
                serialization_format="cdr",
            ),
            rosbag2_py._storage.TopicMetadata(
                name="_shape_info",
                type="std_msgs/msg/String",  #'std_msgs/msg/String',
                serialization_format="cdr",
            ),
            rosbag2_py._storage.TopicMetadata(
                name="clock", type="rosgraph_msgs/msg/Clock", serialization_format="cdr"
            ),
            rosbag2_py._storage.TopicMetadata(
                name="cmd_vel",
                type="geometry_msgs/msg/Twist",
                serialization_format="cdr",
            ),
            rosbag2_py._storage.TopicMetadata(
                name="_ros_status",
                type="std_msgs/msg/String",
                serialization_format="cdr",
            ),
            rosbag2_py._storage.TopicMetadata(
                name="/_ros_node_topic_info",
                type="std_msgs/msg/String",
                serialization_format="cdr",
            ),
            rosbag2_py._storage.TopicMetadata(
                name="tcp_pose_raw",
                type="geometry_msgs/msg/PoseStamped",
                serialization_format="cdr",
            ),
            rosbag2_py._storage.TopicMetadata(
                name="joint_states",
                type="sensor_msgs/msg/JointState",
                serialization_format="cdr",
            ),
            rosbag2_py._storage.TopicMetadata(
                name="/_fruit_info",
                type="std_msgs/msg/String",
                serialization_format="cdr",
            ),
            rosbag2_py._storage.TopicMetadata(
                name="/_bad_fruit_positions",
                type="std_msgs/msg/String",
                serialization_format="cdr",
            ),
        ]

        for topic in self.topic_info_list:
            self.writer.create_topic(topic)

        self.odom = Odometry()
        self.shape_info = String()
        self.gazebo_clock = Clock()
        self.last_gazebo_write_time = 0
        self.twist_stamped_msg = TwistStamped()
        self.last_twist_write_time = 0
        self.ros_status = Contacts()
        self.tf_msg = TFMessage()
        self.last_tf_write_time = 0
        self.tcp_pose_msg = PoseStamped()
        self.last_tcp_pose_write_time = 0
        self.joint_state_msg = JointState()
        self.last_joint_state_write_time = 0
        self.fruit_info_msg = String()
        self.last_fruit_info_write_time = 0
        self.bad_fruit_position_msg = String()
        self.last_bad_fruit_position_write_time = 0
        self.detection_status = String()




        # QoS profiles
        self.reliable_qos = QoSProfile(
            depth=500,
            reliability=QoSReliabilityPolicy.RELIABLE,
            history=QoSHistoryPolicy.KEEP_LAST,
        )
        self.best_effort_qos = QoSProfile(
            depth=500,
            reliability=QoSReliabilityPolicy.BEST_EFFORT,
            history=QoSHistoryPolicy.KEEP_LAST,
        )

        self.odom_sub = self.create_subscription(
            Odometry, "odom", self.record_odom, qos_profile=self.reliable_qos
        )
        self.detection_status_sub = self.create_subscription(
            String,
            "detection_status",
            self.record_detection_status,
            qos_profile=self.reliable_qos,
        )
        self.shape_info_sub = self.create_subscription(
            String,
            "_shape_info",
            self.record_shape_info,
            qos_profile=self.reliable_qos,
        )
        self.clock_sub = self.create_subscription(
            Clock, "clock", self.record_clock, qos_profile=self.best_effort_qos
        )
        self.cmd_vel_sub = self.create_subscription(
            Twist, "cmd_vel", self.record_cmd_vel, qos_profile=self.reliable_qos
        )
        self.ros_status_sub = self.create_subscription(
            Contacts,
            "_ros_status",
            self.record_ros_status,
            qos_profile=self.reliable_qos,
        )
        self.create_subscription(
            JointState,
            "joint_states",
            self._joint_states_callback,
            qos_profile=self.reliable_qos,
        )
        self.create_subscription(
            PoseStamped,
            "tcp_pose_raw",
            self._tcp_pose_callback,
            qos_profile=self.reliable_qos,
        )
        self.create_subscription(
            TFMessage, "tf", self._tf_callback, qos_profile=self.reliable_qos
        )
        self.create_subscription(
            String,
            "/_fruit_info",
            self._fruit_info_callback,
            qos_profile=self.reliable_qos,
        )
        self.create_subscription(
            String,
            "/_bad_fruit_positions",
            self._bad_fruit_positions_callback,
            qos_profile=self.reliable_qos,
        )

        self.create_timer(0.1, self.timer_loop)
        self.create_timer(5.0, self.node_topic_status_callback)

    # Callbacks
    def _tf_callback(self, msg):
        self.tf_msg = msg
        current_time = self.get_clock().now().nanoseconds
        if current_time - self.last_tf_write_time > 1e8:  # 100 ms
            self.last_tf_write_time = current_time
            self._write_bag("tf", msg)

    def _tcp_pose_callback(self, msg):
        self.tcp_pose_msg = msg
        current_time = self.get_clock().now().nanoseconds
        if current_time - self.last_tcp_pose_write_time > 1e8:  # 100 ms
            self.last_tcp_pose_write_time = current_time
            self._write_bag("tcp_pose_raw", msg)

    def _joint_states_callback(self, msg):
        self.joint_state_msg = msg
        current_time = self.get_clock().now().nanoseconds
        if current_time - self.last_joint_state_write_time > 1e8:  # 100 ms
            self.last_joint_state_write_time = current_time
            self._write_bag("joint_states", msg)

    def _fruit_info_callback(self, msg):
        self.fruit_info_msg = msg
        current_time = self.get_clock().now().nanoseconds
        if current_time - self.last_fruit_info_write_time > 1e8:  # 100 ms
            self.last_fruit_info_write_time = current_time
            self._write_bag("/_fruit_info", msg)

    def _bad_fruit_positions_callback(self, msg):
        self.bad_fruit_position_msg = msg
        current_time = self.get_clock().now().nanoseconds
        if current_time - self.last_bad_fruit_position_write_time > 1e8:  # 100 ms
            self.last_bad_fruit_position_write_time = current_time
            self._write_bag("/_bad_fruit_positions", msg)
    # Helper to write messages
    def _write_bag(self, topic_name, msg):
        timestamp = self.get_clock().now().nanoseconds
        self.writer.write(topic_name, serialize_message(msg), timestamp)

    def record_odom(self, msg):
        self.odom = msg

    def record_detection_status(self, msg):
        self.detection_status = msg
        self._write_bag("detection_status", self.detection_status)

    def record_shape_info(self, msg):
        self.shape_info = msg
        self._write_bag("_shape_info", self.shape_info)

    def record_clock(self, msg):
        self.gazebo_clock = msg
        current_time = self.get_clock().now().nanoseconds
        if current_time - self.last_gazebo_write_time > 1e8:  # 100 ms
            self.last_gazebo_write_time = current_time
            self._write_bag("clock", self.gazebo_clock)

    def record_cmd_vel(self, msg):
        self.twist_stamped_msg.twist = msg
        self.twist_stamped_msg.header.stamp = self.get_clock().now().to_msg()
        self._write_bag("cmd_vel", self.twist_stamped_msg.twist)

    def record_ros_status(self, msg):
        self.ros_status = msg
        # self.writer.write('_ros_status', serialize_message(self.ros_status), self.get_clock().now().nanoseconds)

    def node_topic_status_callback(self):
        try:
            node_names = get_node_names(node=self, include_hidden_nodes=True)
            node_topic_info = {"nodes": {}}
            for node in node_names:
                # abs_node_name = get_absolute_node_name(str(node))
                # print("Node:", node, str(node), "Absolute Name:", abs_node_name)
                publishers = get_topics(
                    remote_node_name=node.full_name,
                    func=self.get_publisher_names_and_types_by_node,
                    include_hidden_topics=True,
                )
                node_topic_info["nodes"][f"{node.full_name}"] = dict(publishers)
            # Properly wrap JSON string into a ROS 2 String message before serialization
            node_topic_info_msg = String()
            node_topic_info_msg.data = json.dumps(node_topic_info)
            self.writer.write(
                "/_ros_node_topic_info",
                serialize_message(node_topic_info_msg),
                self.get_clock().now().nanoseconds,
            )
            # print(node_topic_info_msg.data)
            # Also publish for any live consumers
            # self.ros_info_pub.publish(node_topic_info_msg)
        except Exception as e:
            logging.exception(
                f"Exception in recording node topic info data: {e}", exc_info=True
            )
            # self.get_logger().info(f"Exception in recording node topic info data: {e}")
            # SystemExit

    def timer_loop(self):
        try:
            self.ebot_base_link_msg = self.tf_buffer.lookup_transform(
                "odom", "ebot_base_link", rclpy.time.Time()
            )
            self.odom_msg = PoseStamped()
            self.odom_msg.header = self.odom.header
            self.odom_msg.pose = self.odom.pose.pose

            self.get_logger().info(
                "\033[92m DATA COLLECTION STARTED FOR TASK 3B \033[00m", once=True
            )

            self.writer.write(
                "odom",
                serialize_message(self.odom_msg),
                self.get_clock().now().nanoseconds,
            )
            # try:
            #     self._write_bag("_shape_info", self.shape_info)
            # except:
            #     pass

            # try:
            #     self._write_bag(
            #         "clock",
            #         self.gazebo_clock)
            # except:
            #     pass
                
            # try:
            #     self._write_bag(
            #         "tcp_pose_raw",
            #         self.tcp_pose_msg)
            # except:
            #     pass

            # try:
            #     if (
            #         abs(
            #             self.twist_stamped_msg.header.stamp.sec
            #             - self.odom.header.stamp.sec
            #         )
            #         < 0.1
            #     ):
            #         self._write_bag(
            #             "cmd_vel",
            #             self.twist_stamped_msg,
            #         )
            # except:
            #     pass

            # try:
            #     self._write_bag(
            #         "joint_states",
            #         self.joint_state_msg)
            # except:
            #     pass

            # try:
            #     self._write_bag(
            #         "/_fruit_info",
            #         self.fruit_info_msg)
            # except:
            #     pass

            # try:
            #     self._write_bag(
            #         "/_bad_fruit_positions",
            #         self.bad_fruit_position_msg)
            # except:
            #     pass

            # try:
            #     self._write_bag(
            #         "tf",
            #         self.tf_msg)
            # except:
            #     pass

            try:
                if (
                    abs(self.ros_status.header.stamp.sec - self.odom.header.stamp.sec)
                    < 0.1
                ):
                    collision_info = {
                        "time": self.ros_status.header.stamp.sec
                        + self.ros_status.header.stamp.nanosec * 1e-9,
                        "collisions": {},
                    }
                    for id, collision in enumerate(self.ros_status.contacts):
                        collision_info["collisions"][f"{id}"] = {
                            "collision1": collision.collision1.name,
                            "collision2": collision.collision2.name,
                        }
                    # Properly wrap JSON string into a ROS 2 String message before serialization
                    collision_info_msg = String()
                    collision_info_msg.data = json.dumps(collision_info)

                    self._write_bag(
                        "_ros_status",
                        collision_info_msg,
                    )
            except Exception as e:
                self.get_logger().info(
                    f"Exception in recording collision data: {e}", once=True
                )
                SystemExit
        except Exception as e:
            self.get_logger().info(f"Exception in recording data: {e}", once=True)
            self.get_logger().warning("WAITING FOR DATA", once=True)
            pass


def evaluate():
    result = {}
    console = Console()
    rclpy.init()
    sbr = SimpleBagRecorder()
    try:
        rclpy.spin(sbr)
    except KeyboardInterrupt or RuntimeError:
        console.print(f"[green]\nExiting Gracefully[/green]")
    # rclpy.shutdown()
    result["generate"] = False
    return result



"""
*****************************************************************************************
*
*   ===================================================
*       CropDrop Bot (CB) Theme [eYRC 2025-26]
*   ===================================================
*
*  This script is intended to check the versions of the installed
*  software/libraries in Task 0 of CropDrop Bot (CB) Theme [eYRC 2025-26].
*
*  Filename:		evaluator.py
*  Created:			14/08/2025
*  Last Modified:	14/08/2025
*  Author:			e-Yantra Team
*  
*  This software is made available on an "AS IS WHERE IS BASIS".
*  Licensee/end user indemnifies and will keep e-Yantra indemnified from
*  any and all claim(s) that emanate from the use of the Software or
*  breach of the terms of this agreement.
*  
*  e-Yantra - An MHRD project under National Mission on Education using ICT (NMEICT)
*
*****************************************************************************************
"""

# ===== Standard imports =====
import os
import sys
import platform
import shutil
import subprocess
from pathlib import Path
import psutil

# ===== Third-party imports =====
import distro
from rich.console import Console
from rich.prompt import Confirm
from eyantra_autoeval.utils.common import run_shell_command

# ===== Helper functions =====
def check_gcc_or_gplusplus():
    return shutil.which("gcc") is not None or shutil.which("g++") is not None

def check_python3_version():
    return sys.version_info.major >= 3

def find_file(filename, search_paths):
    """Search for a file in given paths recursively"""
    for base_path in search_paths:
        base_path = Path(base_path).expanduser()
        if base_path.exists():
            for root, dirs, files in os.walk(base_path):
                if filename in files:
                    return True
    return False

def Check_running(app):
    for proc in psutil.process_iter(['name']):
        if proc.info['name'] and app  in proc.info['name']:
            return True
    return False

# ===== Main Evaluator =====
def evaluate():
    result = {}
    console = Console()
    result["generate"] = False

    # ===== OS Information =====
    with console.status("[bold green]Gathering system information..."):
        distribution = {
            "machine": platform.machine(),
            "version": distro.version(best=True),
            "name": distro.name(),
        }
        result["os_info"] = distribution
        result["os_type"] = platform.system()

    # ===== OS Compatibility Check =====
    self_os_declaration = True
    os_name = result["os_info"]["name"]
    os_version = result["os_info"]["version"]
    os_machine = result["os_info"]["machine"]
    os_type = result["os_type"]

    # WSL detection using uname -r
    uname = run_shell_command("uname -r").split("-")
    is_wsl = "Microsoft" in uname[-1] or "WSL" in uname[-1]

    # Case 1: Ubuntu 22.04 or 24.04 ()
    if os_name == "Ubuntu" and os_machine == "x86_64" and ("22.04" in os_version or "24.04" in os_version):
        if is_wsl:
            console.print("[red]Ubuntu in WSL is not supported. Please use baremetal Ubuntu.[/red]")
            result["generate"] = False
        else:
            console.print(f"[yellow]You are using Ubuntu {os_version} (baremetal)[/yellow]")
            result["wsl"] = False

    # Case 2: Windows (baremetal only)
    elif os_type == "Windows" and (os_machine == "x86_64" or os_machine == "AMD64"):
        console.print(f"[yellow]You are using Windows {os_version}[/yellow]")
        result["wsl"] = False

    # Unsupported OS
    else:
        console.print(
            f"""[yellow]You are using [bold blue]{os_name} {os_version} ({os_machine})[/bold blue].  
    We officially support Ubuntu 22.04 and 24.04 or Windows. You may continue, but e-Yantra will not help with OS-specific installation issues.[/yellow]"""
        )
        self_os_declaration = Confirm.ask(
            "Do you accept to take ownership of your setup and not seek OS-specific help from e-Yantra?"
        )

    result["self-os-declaration"] = self_os_declaration

    # ===== Installation Checks =====
    with console.status("[bold green]Checking required tools and software..."):
        result["gcc_or_g++_installed"] = check_gcc_or_gplusplus()
        result["python3_version_ok"] = check_python3_version()
        result["coppeliasim_installed"] = Check_running("coppeliaSim")
        result["stm32cubeide_installed"] = Check_running("stm32cubeide")


    # ===== Summary =====
    console.print("\n[bold underline]Installation Check Results:[/bold underline]")
    for key, value in {
        "GCC or G++": result["gcc_or_g++_installed"],
        "Python 3 Version OK": result["python3_version_ok"],
        "CoppeliaSim Installed": result["coppeliasim_installed"],
        "STM32CubeIDE Installed": result["stm32cubeide_installed"],
    }.items():
        status = "[green]✓[/green]" if value else "[red]✗[/red]"
        console.print(f"{status} {key}")

    # ===== Final Verdict =====
    if all([
        result["gcc_or_g++_installed"],
        result["python3_version_ok"],
        result["coppeliasim_installed"],
        result["stm32cubeide_installed"],
        self_os_declaration
    ]):
        console.print("\n[bold green]✅ All checks passed. Task 0 Output is correct![/bold green]")
        console.print("[bold yellow]JSON File Created.[/bold yellow]")
        result["generate"] = True
    else:
        console.print("\n[bold red]❌ Task 0 Output is incorrect! Please verify installations.[/bold red]")
        console.print("[bold yellow] JSON File Not Created.[/bold yellow]")
        
        result["generate"] = False

    return result


# if __name__ == "__main__":
#     evaluate()




import shutil
from rich.console import Console

import rclpy
from rclpy.node import Node
from rclpy.serialization import serialize_message
from rclpy.qos import QoSProfile, QoSReliabilityPolicy, QoSHistoryPolicy

import rosbag2_py

from rosgraph_msgs.msg import Clock
from tf2_msgs.msg import TFMessage
from sensor_msgs.msg import JointState
from geometry_msgs.msg import PoseStamped
from std_msgs.msg import String


class SimpleBagRecorder(Node):
    def __init__(self):
        super().__init__('simple_bag_recorder')
        self.console = Console()
        self.writer = rosbag2_py.SequentialWriter()

        # Remove existing bag
        try:
            shutil.rmtree("my_bag")
        except FileNotFoundError:
            pass

        # Open rosbag2 writer
        storage_options = rosbag2_py._storage.StorageOptions(uri='my_bag', storage_id='sqlite3')
        converter_options = rosbag2_py._storage.ConverterOptions('', '')
        self.writer.open(storage_options, converter_options)

        # Define topics metadata
        self.topics_info = [
            rosbag2_py._storage.TopicMetadata(name='tf', type='tf2_msgs/msg/TFMessage', serialization_format='cdr'),
            rosbag2_py._storage.TopicMetadata(name='tcp_pose_raw', type='geometry_msgs/msg/PoseStamped', serialization_format='cdr'),
            rosbag2_py._storage.TopicMetadata(name='joint_states', type='sensor_msgs/msg/JointState', serialization_format='cdr'),
            rosbag2_py._storage.TopicMetadata(name='/clock', type='rosgraph_msgs/msg/Clock', serialization_format='cdr'),
            rosbag2_py._storage.TopicMetadata(name='/_fruit_info', type='std_msgs/msg/String', serialization_format='cdr'),
            rosbag2_py._storage.TopicMetadata(name='/_bad_fruit_positions', type='std_msgs/msg/String', serialization_format='cdr')
            
        ]
        
        # Create topics
        for topic in self.topics_info:
            self.writer.create_topic(topic)

        # QoS profiles
        self.reliable_qos = QoSProfile(depth=500, reliability=QoSReliabilityPolicy.RELIABLE, history=QoSHistoryPolicy.KEEP_LAST)
        self.best_effort_qos = QoSProfile(depth=500, reliability=QoSReliabilityPolicy.BEST_EFFORT, history=QoSHistoryPolicy.KEEP_LAST)

        # Subscriptions
        self.create_subscription(Clock, '/clock', self._clock_callback, self.best_effort_qos)
        self.create_subscription(JointState, 'joint_states', self._joint_states_callback, self.reliable_qos)
        self.create_subscription(PoseStamped, 'tcp_pose_raw', self._tcp_pose_callback, self.reliable_qos)
        self.create_subscription(TFMessage, 'tf', self._tf_callback, self.reliable_qos)
        self.create_subscription(String, '/_fruit_info', self._fruit_info_callback, self.reliable_qos)
        self.create_subscription(String, '/_bad_fruit_positions', self._bad_fruit_positions_callback, self.reliable_qos)
        self.get_logger().info("\033[92mDATA COLLECTION STARTED\033[00m")

    # Callbacks
    def _tf_callback(self, msg):
        self._write_bag('tf', msg)

    def _tcp_pose_callback(self, msg):
        self._write_bag('tcp_pose_raw', msg)

    def _joint_states_callback(self, msg):
        self._write_bag('joint_states', msg)

    def _clock_callback(self, msg):
        self._write_bag('/clock', msg)

    def _fruit_info_callback(self, msg):
        self._write_bag('/_fruit_info', msg)

    def _bad_fruit_positions_callback(self, msg):
        self._write_bag('/_bad_fruit_positions', msg)

    # Helper to write messages
    def _write_bag(self, topic_name, msg):
        timestamp = self.get_clock().now().nanoseconds
        self.writer.write(topic_name, serialize_message(msg), timestamp)


# Keep your evaluate() function exactly as it is
def evaluate():
    result = {}
    console = Console()
    rclpy.init()
    sbr = SimpleBagRecorder()
    try:
        rclpy.spin(sbr)
    except KeyboardInterrupt:
        console.print(f"[green]\nExiting Gracefully[/green]")
    # rclpy.shutdown()
    result["generate"] = False
    return result



import rclpy
import shutil
from rclpy.node import Node
from rclpy.serialization import serialize_message
import rosbag2_py
import tf2_ros
from rich.console import Console
from nav_msgs.msg import Odometry
from geometry_msgs.msg import PoseStamped, Twist
from ros_gz_interfaces.msg import Contacts
from std_msgs.msg import String
from rosgraph_msgs.msg import Clock
import json
import logging
from ros2node.api import get_absolute_node_name, get_node_names, get_topics
from ros2topic.api import get_topic_names_and_types


class SimpleBagRecorder(Node):
    def __init__(self):
        super().__init__('simple_bag_recorder')
        self.tf_buffer = tf2_ros.buffer.Buffer()
        self.r = tf2_ros.TransformBroadcaster(self)
        self.listener = tf2_ros.TransformListener(self.tf_buffer, self)
        self.writer = rosbag2_py.SequentialWriter()
        try:
            shutil.rmtree("my_bag")
        except:
            pass
        storage_options = rosbag2_py._storage.StorageOptions(
            uri='my_bag',
            storage_id='sqlite3')
        converter_options = rosbag2_py._storage.ConverterOptions('', '')
        self.writer.open(storage_options, converter_options)

        topic_info_tf = rosbag2_py._storage.TopicMetadata(
            name='tf',
            type= 'geometry_msgs/msg/TransformStamped', #'std_msgs/msg/String',
            serialization_format='cdr')
        topic_info_odom = rosbag2_py._storage.TopicMetadata(
            name='odom',
            type= 'geometry_msgs/msg/PoseStamped', #'std_msgs/msg/String',
            serialization_format='cdr')
        topic_info_detection_status = rosbag2_py._storage.TopicMetadata(
            name='detection_status',
            type= 'std_msgs/msg/String', #'std_msgs/msg/String',
            serialization_format='cdr')
        topic_info_shape_info = rosbag2_py._storage.TopicMetadata(
            name='_shape_info',
            type= 'std_msgs/msg/String', #'std_msgs/msg/String',
            serialization_format='cdr')
        topic_info_clock = rosbag2_py._storage.TopicMetadata(
            name='clock',
            type= 'rosgraph_msgs/msg/Clock',
            serialization_format='cdr')
        topic_info_cmd_vel = rosbag2_py._storage.TopicMetadata(
            name='cmd_vel',
            type= 'geometry_msgs/msg/Twist',
            serialization_format='cdr')
        topic_info_ros_status = rosbag2_py._storage.TopicMetadata(
            name='_ros_status',
            type= 'std_msgs/msg/String',
            serialization_format='cdr')
        topic_info_ros_node_topic_info = rosbag2_py._storage.TopicMetadata(
            name='/_ros_node_topic_info',
            type= 'std_msgs/msg/String',
            serialization_format='cdr')
        # topic_info_ros_info = rosbag2_py._storage.TopicMetadata(
        #     name='_ros_info',
        #     type= 'std_msgs/msg/String',
        #     serialization_format='cdr')

        self.writer.create_topic(topic_info_tf)
        self.writer.create_topic(topic_info_odom)
        self.writer.create_topic(topic_info_detection_status)
        self.writer.create_topic(topic_info_shape_info)
        self.writer.create_topic(topic_info_clock)
        self.writer.create_topic(topic_info_cmd_vel)
        self.writer.create_topic(topic_info_ros_status)
        self.writer.create_topic(topic_info_ros_node_topic_info)
        # self.writer.create_topic(topic_info_ros_info)

        self.odom=Odometry()
        self.shape_info=String()
        self.gazebo_clock=Clock()
        self.twist = Twist()
        self.ros_status = Contacts()
        self.odom_sub = self.create_subscription(Odometry, 'odom', self.record_odom, 10)
        self.detection_status_sub = self.create_subscription(String, 'detection_status', self.record_detection_status, 10)
        self.shape_info_sub = self.create_subscription(String, '_shape_info', self.record_shape_info, 10)
        self.clock_sub = self.create_subscription(Clock, 'clock', self.record_clock, 10)
        self.cmd_vel_sub = self.create_subscription(Twist, 'cmd_vel', self.record_cmd_vel, 10)
        self.ros_status_sub = self.create_subscription(Contacts, '_ros_status', self.record_ros_status, 10) ## Collision Topic
        # self.ros_info_pub = self.create_publisher(String, '_ros_info', 10) ## Collision republishing Topic
        self.create_timer(0.1, self.timer_loop)
        self.create_timer(5.0, self.node_topic_status_callback)

    def record_odom(self, msg):
        self.odom=msg

    def record_detection_status(self, msg):
        self.detection_status=msg
        self.writer.write('detection_status', serialize_message(self.detection_status), self.get_clock().now().nanoseconds)

    def record_shape_info(self, msg):
        self.shape_info=msg
        self.writer.write('_shape_info', serialize_message(self.shape_info), self.get_clock().now().nanoseconds)

    def record_clock(self, msg):
        self.gazebo_clock=msg
        # self.writer.write('clock', serialize_message(self.gazebo_clock), self.get_clock().now().nanoseconds)

    def record_cmd_vel(self, msg):
        self.twist=msg
        self.writer.write('cmd_vel', serialize_message(self.twist), self.get_clock().now().nanoseconds)

    def record_ros_status(self, msg):
        self.ros_status=msg
        # self.writer.write('_ros_status', serialize_message(self.ros_status), self.get_clock().now().nanoseconds)

    def node_topic_status_callback(self):
        try:
            node_names = get_node_names(node=self, include_hidden_nodes=True)
            node_topic_info = {"nodes": {}}
            for node in node_names:
                # abs_node_name = get_absolute_node_name(str(node))
                # print("Node:", node, str(node), "Absolute Name:", abs_node_name)
                publishers = get_topics(remote_node_name=node.full_name, func=self.get_publisher_names_and_types_by_node, include_hidden_topics=True)
                node_topic_info["nodes"][f"{node.full_name}"] = dict(publishers)
            # Properly wrap JSON string into a ROS 2 String message before serialization
            node_topic_info_msg = String()
            node_topic_info_msg.data = json.dumps(node_topic_info)
            self.writer.write('/_ros_node_topic_info', serialize_message(node_topic_info_msg), self.get_clock().now().nanoseconds)
            # print(node_topic_info_msg.data)
            # Also publish for any live consumers
            # self.ros_info_pub.publish(node_topic_info_msg)
        except Exception as e:
            logging.exception(f"Exception in recording node topic info data: {e}", exc_info=True)
            # self.get_logger().info(f"Exception in recording node topic info data: {e}")
            # SystemExit
        
        
    def timer_loop(self):
        try:
            self.ebot_base_link_msg = self.tf_buffer.lookup_transform('odom', "ebot_base_link", rclpy.time.Time())
            self.odom_msg=PoseStamped()
            self.odom_msg.header=self.odom.header
            self.odom_msg.pose=self.odom.pose.pose

            self.get_logger().info("\033[92m DATA COLLECTION STARTED FOR TASK 2A \033[00m", once=True)
            self.writer.write('tf', serialize_message(self.ebot_base_link_msg), self.get_clock().now().nanoseconds)
            self.writer.write('odom', serialize_message(self.odom_msg), self.get_clock().now().nanoseconds)
            # try:
            #     self.writer.write('detection_status', serialize_message(self.detection_status), self.get_clock().now().nanoseconds)
            # except:
            #     pass
            try:
                self.writer.write('_shape_info', serialize_message(self.shape_info), self.get_clock().now().nanoseconds)
            except:
                pass
            try:
                self.writer.write('clock', serialize_message(self.gazebo_clock), self.get_clock().now().nanoseconds)
            except:
                pass
            # try:
            #     self.writer.write('cmd_vel', serialize_message(self.twist), self.get_clock().now().nanoseconds)
            # except:
            #     pass
            try:
                if abs(self.ros_status.header.stamp.sec-self.odom.header.stamp.sec)<0.1:
                    collision_info = {"time": self.ros_status.header.stamp.sec+self.ros_status.header.stamp.nanosec*1e-9, "collisions": {}}
                    for id, collision in enumerate(self.ros_status.contacts):
                        collision_info["collisions"][f"{id}"] = {
                            "collision1": collision.collision1.name,
                            "collision2": collision.collision2.name,
                        }
                    # Properly wrap JSON string into a ROS 2 String message before serialization
                    collision_info_msg = String()
                    collision_info_msg.data = json.dumps(collision_info)
                    self.writer.write('_ros_status', serialize_message(collision_info_msg), self.get_clock().now().nanoseconds)
                    # Also publish for any live consumers
                    # self.ros_info_pub.publish(collision_info_msg)
            except Exception as e:
                # logging.exception(f"Exception in recording collision data: {e}", exc_info=True)
                self.get_logger().info(f"Exception in recording collision data: {e}", once=True)
                SystemExit
        except Exception as e:
            self.get_logger().info(f"Exception in recording data: {e}", once=True)
            self.get_logger().warning("WAITING FOR DATA", once=True)
            pass


def evaluate():
    result = {}
    console = Console()
    rclpy.init()
    sbr = SimpleBagRecorder()
    try:
        rclpy.spin(sbr)
    except KeyboardInterrupt or RuntimeError: 
        console.print(f"[green]\nExiting Gracefully[/green]")
    # rclpy.shutdown()
    result["generate"] = False
    return result


"""
evaluator.py

This file is used to evaluate task 2A of FPGA theme.
It generates the dump_txt.txt file which is used in the simulation.

"""
import os
import random

def toBinary(a):
    l,m=[],[]
    for i in a:
        l.append(ord(i))
    for i in l:
        m.append(bin(i)[2:].zfill(8))
    return m

def write_to_file(filepath, data, module,changed_indices):
    with open(filepath, 'w') as f:
        for parity,i in enumerate(data):
            f.write('0\n')
            for j in i:
                f.write(j)
                f.write('\n') 
            if module == "rx":
                if parity in changed_indices:
                    parity_bit = int(i.count('1')%2 == 0)
                else:
                    parity_bit = int(i.count('1')%2 != 0)
                f.write(str(parity_bit))
                f.write('\n')
            f.write('1\n')

def evaluate():
    directory_path = "/simulation/modelsim/"
    result = {}

    try:
        full_path = os.getcwd() + directory_path
        if os.path.isdir(full_path):
            module = input("Enter the module (tx/rx): ")
            if module == "tx" or module == "rx":
                text = input("Enter the text: ")
                length = len(text)

                if length <= 10:
                    changed_indices = []
                    num_letters = random.choice([1,1+length%4])
                    if module == "rx":
                        random_indices = random.sample(range(length), num_letters)
                        string_list = list(text)
                        for i in random_indices:
                            string_list[i] = chr(ord(string_list[i]) - 2)
                        modified_text = ''.join(string_list)
                        changed_indices = random_indices

                    data = toBinary(text)
                
                    write_to_file(full_path + 'data.txt', data, module,changed_indices)

                    print("File data.txt has been created.")

                else:
                    print("Entered text size is greater than 10!")
            else:
                print("Entered module name is not listed!")
        else:
            print(f"The directory '{directory_path}' does not exist.")

    except Exception as e:
        print(f"An error occurred: {e}")

    result["generate"] = False
    return result

if __name__ == "__main__":
    evaluate()



'''
To evaluate Task 1B of CL eYRC23-24
Name: Jaison J.
Credit: e-Yantra, IIT Bombay
CDate: 12-09-2023
'''

import rclpy
import shutil
from rclpy.node import Node
from rclpy.serialization import serialize_message
import rosbag2_py
from rich.console import Console
from gazebo_msgs.msg import LinkStates
from std_msgs.msg import String

class SimpleBagRecorder(Node):
    def __init__(self):
        super().__init__('simple_bag_recorder')
        self.writer = rosbag2_py.SequentialWriter()
        try:
            shutil.rmtree("my_bag")
        except:
            pass
        storage_options = rosbag2_py._storage.StorageOptions(
            uri='my_bag',
            storage_id='sqlite3')
        converter_options = rosbag2_py._storage.ConverterOptions('', '')

        self.writer.open(storage_options, converter_options)
        topic_info = rosbag2_py._storage.TopicMetadata(
            name='/_makeit/link_states',
            type= 'gazebo_msgs/msg/LinkStates',
            serialization_format='cdr')
        topic2_info = rosbag2_py._storage.TopicMetadata(
            name='/_sprunner',
            type= 'std_msgs/msg/String',
            serialization_format='cdr')
        self.writer.create_topic(topic_info)
        self.writer.create_topic(topic2_info)
        # self.create_timer(0.5, self.topic_callback)

        self.create_subscription(LinkStates, '/_makeit/link_states', self.topic_callback, 10)
        self.create_subscription(String, '/_sprunner', self.topic2_callback, 10)

        self.get_logger().info("\033[92m DATA COLLECTION STARTED \033[00m", once=True)

    def topic_callback(self, data):
        self.writer.write('/_makeit/link_states', serialize_message(data), self.get_clock().now().nanoseconds)
    
    def topic2_callback(self, data):
        self.writer.write('/_sprunner', serialize_message(data), self.get_clock().now().nanoseconds)

def evaluate():
    result = {}
    console = Console()
    rclpy.init()
    sbr = SimpleBagRecorder()
    try:
        rclpy.spin(sbr)
    except KeyboardInterrupt:
        console.print(f"[green]\nExiting Gracefully[/green]")
    # rclpy.shutdown()
    result["generate"] = False
    return result


import rclpy
import shutil
from rclpy.node import Node
from rclpy.serialization import serialize_message
import rosbag2_py
from rich.console import Console
from gazebo_msgs.msg import LinkStates
from std_msgs.msg import String
from rosgraph_msgs.msg import Clock
from rclpy.qos import QoSProfile, QoSReliabilityPolicy, QoSHistoryPolicy
import rclpy
import shutil
from rclpy.node import Node
from rclpy.serialization import serialize_message
import rosbag2_py
import tf2_ros
from rich.console import Console
from tf2_msgs.msg import TFMessage
from gazebo_msgs.msg import LinkStates
from std_msgs.msg import String
from sensor_msgs.msg import JointState
from geometry_msgs.msg import PoseStamped

class SimpleBagRecorder(Node):
    def __init__(self):
        super().__init__('simple_bag_recorder')
        self.console = Console()
        self.writer = rosbag2_py.SequentialWriter()

        # Delete old bag
        try:
            shutil.rmtree("my_bag")
        except FileNotFoundError:
            pass

        # Open storage
        storage_options = rosbag2_py._storage.StorageOptions(
            uri='my_bag',
            storage_id='sqlite3'
        )
        converter_options = rosbag2_py._storage.ConverterOptions('', '')
        self.writer.open(storage_options, converter_options)

        # Create topics
        tcp_topic_info = rosbag2_py._storage.TopicMetadata(
            name='tcp_pose_raw',
            type='geometry_msgs/msg/PoseStamped',
            serialization_format='cdr'
        )
        joint_topic_info = rosbag2_py._storage.TopicMetadata(
            name='joint_states',
            type='sensor_msgs/msg/JointState',
            serialization_format='cdr'
        )
        clock_topic_info = rosbag2_py._storage.TopicMetadata(
            name='/clock',
            type='rosgraph_msgs/msg/Clock',
            serialization_format='cdr'
        )

        self.writer.create_topic(tcp_topic_info)
        self.writer.create_topic(joint_topic_info)
        self.writer.create_topic(clock_topic_info)

        # QoS profiles
        reliable_qos = QoSProfile(
            depth=500,
            reliability=QoSReliabilityPolicy.RELIABLE,
            history=QoSHistoryPolicy.KEEP_LAST
        )

        best_effort_qos = QoSProfile(
            depth=500,
            reliability=QoSReliabilityPolicy.BEST_EFFORT,
            history=QoSHistoryPolicy.KEEP_LAST
        )

        # Subscriptions
        self.create_subscription(Clock, '/clock', self.clock_callback, best_effort_qos)
        self.create_subscription(JointState, 'joint_states', self.joint_states_callback, reliable_qos)
        self.create_subscription(PoseStamped, 'tcp_pose_raw', self.tcp_pose_callback, 10)

        self.get_logger().info("\033[92m DATA COLLECTION STARTED \033[00m", once=True)

    # Callbacks
    def tcp_pose_callback(self, msg):
        # print("TCP POSE CALLBACK",msg)
        self.writer.write('tcp_pose_raw', serialize_message(msg), self.get_clock().now().nanoseconds)

    def joint_states_callback(self, msg):
        self.writer.write('joint_states', serialize_message(msg), self.get_clock().now().nanoseconds)

    def clock_callback(self, msg):
        self.writer.write('/clock', serialize_message(msg), self.get_clock().now().nanoseconds)

def evaluate():
    result = {}
    console = Console()
    rclpy.init()
    sbr = SimpleBagRecorder()
    try:
        rclpy.spin(sbr)
    except KeyboardInterrupt:
        console.print(f"[green]\nExiting Gracefully[/green]")
    # rclpy.shutdown()
    result["generate"] = False
    return result






'''
*****************************************************************************************
*
*        =================================================
*             Builder Bot Theme (eYRC 2024-25)
*        =================================================
*
*  This script is intended to check the versions of the installed
*  software/libraries in Task 0A of Builder Bot Theme (eYRC 2024-25).
*
*  Filename:			task0a_cardinal.py
*  Created:				16/07/2024
*  Last Modified:		24/08/2024
*  Author:				Paras Padole, e-Yantra Team
*  
*  This software is made available on an "AS IS WHERE IS BASIS".
*  Licensee/end user indemnifies and will keep e-Yantra indemnified from
*  any and all claim(s) that emanate from the use of the Software or 
*  breach of the terms of this agreement.
*  
*  e-Yantra - An MHRD project under National Mission on Education using ICT (NMEICT)
*
*****************************************************************************************
'''


# Import modules
import time
import string
import random
import re
import base64
from datetime import datetime
import os, sys
import platform
# from unittest import result
import cryptocode
sys.path.insert(0, 'pythonAPI')
from zmqRemoteApi import RemoteAPIClient 
from oauth2client.service_account import ServiceAccountCredentials # type: ignore
import gspread # type: ignore

#############################################
# third-party imprts
import distro
from eyantra_autoeval.utils.common import is_docker, run_shell_command
from rich import print
from rich.console import Console
from rich.prompt import Confirm

################################


# scope = ['https://www.googleapis.com/auth/spreadsheets', "https://www.googleapis.com/auth/drive.file", "https://www.googleapis.com/auth/drive"]

# credentials = {
# 		"type": "service_account",
# 		"project_id": "bb-logs-431613",
# 		"private_key_id": "a4506d44024339472c6cae03bddb65257200038b",
# 		"private_key": "-----BEGIN PRIVATE KEY-----\nMIIEvQIBADANBgkqhkiG9w0BAQEFAASCBKcwggSjAgEAAoIBAQDA2GeVj68LVk+O\nsEDyOBAIhRH/0BFaD/rf2Swws+5G3EwC6qddOThCkMNsT4CuzpCLUeC9Na+bZCqZ\ngqv3+ygHCNFzCbVhtAu0/L5BlaYAtSOlwdayEJRcA4d1zyIZ9cvzDne8DKiMzcnU\nbQ1UsDCn5gET0GiHPmIYf7UJHc91TKRPa+echb+UPonL0q/BTXnpiNDDKBDPNdM3\nSszl1Hx0oP+SWiWqsaPdKTUjJzRSBm8Sh3TJrvfENehDwoAg6O+UXbTvy7Jx/EGU\nvmbss0zTD3239KihiYa1flQdAYWZv5E8d9U2YSrye+r8+YLbOMqXaPx/xzXPlrfK\n3wp0YzjBAgMBAAECggEABVwGTfNpMpwWmDE5DtpExM3Y9RZ14v5Cc0XsetqHTuAY\nuzSFC/vW/s+Z5MjVW1ZZkUuoXq3PKHcJUYyehaTs5PwnYQZ2LGXV9PTYv0ceztJ5\n8AWB918rVl7RPRKRgK3yefnvFTJ3XTlraGxS9GV0prfnVAME4qxWytZCxGL4FOsM\n40sS6+V5w10KkS0hyBJU9I6AjZMGcVI96hLQtMlFQOQV62EntrV5o0++Bs6lGczp\nrFt/0u+hazMUlDzY3iKvpgayreO5X7qoJJm6dDWSo6Sfq4wlKA3VveUG+cM3t72R\nRE9eljYOCfw6QD9ORa8XBZtNfN9aGXZIoqKbU92TaQKBgQDlNn30gmczKdwkSRQo\nCt/pPU3+CZWhu5rCnXfeHbfzU9u5GrWH40llgQDjgWgxZqtu/uWZyPuo6AtVEwUy\nkh6HqdzdDx42rhG1GrTbAIiSOn606FG8kQjFps8yqIfVWlL9awiTnCppy2O5QFU0\n9UDDzJubFWkdhR5b4TFft11lKQKBgQDXYeHHduYcjtCrlL4CTI7WWaAqA+VknAES\nxS/+CPZjMOv0OyjYnppI03gEiA2y70n/P4NmdUcwvDnJK7mdvuwrRcvsKF+UeV2y\ng0v3xUzaFaxyasfFZuRCYv7pr1cG/hRcmRI5fd9dlS4xWpkXqe/Yc8oU8pGKkbY9\nz4lf09LR2QKBgQCIUgVNIzU/X5j216OuQPF0ZSp6eLbOTqY3MrH0nxYlGG2oRDNM\nkye2v6eIpxERuG8i/2QMN1U82mzK9xnzPqX7p1GdA33DpXkQjcacLVAML8/lxfm+\nvT9LVe8KwOKwSBztbPfX2lv7OaSgq5tBeM9A4/JzpKM0lFQ+7sqPk51vKQKBgGV8\nGYaC36pVIL24OE+dAzC8ylsBuvTNDTRq9VIdpvrV8lgCCB0JnmjyO3rnII1Pcu5y\nXtfIKuMrzY6cq7lIXL+HA68i1uZ+yUdz1jfJH40i6T6AUeERujwNqU8y7y68SZvY\nBF5SkQznXfyjU79yszCqXm3AXhOM1PIK+A/PH2cBAoGAMLztRveOQxy0eU0Cl9KE\nvL4qbH2XYGn1aEHPTY8Dpx4A1varNKhXzUtHGPhVWPB1Qq//jot6+DgClQ6Kda4C\n+Vh8+1GUxv3DihmGcCvwb8BXwpaPhB9qq/2HnoWvq1bSKnRvA5ab8LC1z/DV1DRZ\nfXKQhr8q5KB9QfMmD8IzhzI=\n-----END PRIVATE KEY-----\n",
# 		"client_email": "balancing-builder-bot@bb-logs-431613.iam.gserviceaccount.com",
# 		"client_id": "105579116913873875207",
# 		"auth_uri": "https://accounts.google.com/o/oauth2/auth",
# 		"token_uri": "https://oauth2.googleapis.com/token",
# 		"auth_provider_x509_cert_url": "https://www.googleapis.com/oauth2/v1/certs",
# 		"client_x509_cert_url": "https://www.googleapis.com/robot/v1/metadata/x509/balancing-builder-bot%40bb-logs-431613.iam.gserviceaccount.com",
# 		"universe_domain": "googleapis.com"
# }

# client = gspread.service_account_from_dict(credentials)
# try:
# 	python_cell_update = client.open("BB_Logs").worksheet("task0A")
# 	print(python_cell_update)

# except:
# 	print("\n\t[ERROR] System not able to access internet!\n")
# 	sys.exit()
# values_list = python_cell_update.col_values(1)

# Flags to determine which modules are installed or not
platform_uname = None
conda_flag = None
python_flag = None
cv2_flag = None
numpy_flag = None
matplotlib_flag = None
jupyter_flag = None


# Flags to determine whether the modules installed have correct version or not
conda_env_name_flag = None

# Output file name
file_name = "task0A_output.txt"

# Flag and Global variables to determine whether the CoppeliaSim Remote API Server works fine
coppeliasim_remote_api_flag = None
clientID = 0

left_motor_vel = 0
right_motor_vel = 0




  


# Check whether CoppeliaSim Remote API server is working fine
# def check_coppeliasim_remote_api():

result = {}

def extract_team_id(env_name):
    match = re.search(r'(\d+)', env_name)
    return match.group(0) if match else 'unknown'
 
def evaluate():
	
	# result = {}
	console = Console()
	global result,conda_env_name_flag,coppeliasim_remote_api_flag,left_motor_handle,right_motor_handle,left_motor_vel,right_motor_vel


	try:
			team_id = int(input('\n\tEnter your Team ID (for e.g.: "1234" or "321"): '))
	except ValueError:
			print("\n\t[ERROR] Enter your Team ID which is an integer!\n")
			sys.exit()

	conda_env_name = os.environ['CONDA_DEFAULT_ENV']

	expected_conda_env_name = 'KB_' + str(team_id)

			# Check current Conda environment name is as expected
	if conda_env_name == expected_conda_env_name:
					conda_env_name_flag = 1
	else:
					conda_env_name_flag = 0
					print("\n\t[WARNING] Conda environment name is not found as expected, Make sure it is: KB_%s, re-check the instructions\n" %(str(team_id)))
					result["generate"] = False
					#python_cell_update.update_cell(len(values_list)+1,1,str(team_id) +" Task 0A "+conda_env_name+ " Conda environment name is not found as expected, Make sure it is: BB_"+str(team_id)+", re-check the instructions")
					sys.exit()

	env_name = os.getenv('CONDA_DEFAULT_ENV', 'unknown')
	team_id = extract_team_id(env_name)
				

	platform_uname = platform.uname().system
	conda_env_name = os.getenv('CONDA_DEFAULT_ENV', 'unknown')
	current_time = datetime.now().strftime('%d-%m-%Y %H:%M:%S')

	client = RemoteAPIClient()
	sim = client.getObject('sim')
	defaultIdleFps = sim.getInt32Param(sim.intparam_idle_fps)
	sim.setInt32Param(sim.intparam_idle_fps, 0)

	
	sim.startSimulation()
	return_code=sim.loadModel(str(os.getcwd()+"/Differential_Drive_Robot.ttm"))
	left_motor_handle=sim.getObject('./left_joint')
	right_motor_handle=sim.getObject('./right_joint')


		
	command_msg = ''
	
	print("\n\tCommands to control robot locomotion are:")
	print("\t+-------------------------------------------------------+")

	print("	|	Command to execute	|	Char Input	|")
	print("\t+-------------------------------------------------------+")

	print("	|	Move Forward		| 	'w' OR 'W'	|")
	print("	|	Move Backward		| 	's' OR 'S'	|")
	print("	|	Turn Left		| 	'a' OR 'A'	|")
	print("	|	Turn Right		| 	'd' OR 'D'	|")
	print("	|	Stop			| 	'x' OR 'X'	|")
	print("	|	Quit the program	| 	'q' OR 'Q'	|")
	print("\t+-------------------------------------------------------+")

	while(True):
		
		inp_char = input("\n\tEnter any one of these as input (w, W | a, A | s, S | d, D | x, X | q, Q): ")

		if len(inp_char) == 1:
			
			if inp_char == 'w' or inp_char == 'W':
				left_motor_vel = 2
				right_motor_vel = 2
				command_msg = "Move Forward"
				# print("\n\tmove forward")

			elif inp_char == 'a' or inp_char == 'A':
				left_motor_vel = -2
				right_motor_vel = 2
				command_msg = "Turn Left"
				# print("\n\tturn left")

			elif inp_char == 'd' or inp_char == 'D':
				left_motor_vel = 2
				right_motor_vel = -2
				command_msg = "Turn Right"
				# print("\n\tturn right")

			elif inp_char == 's' or inp_char == 'S':
				left_motor_vel = -2
				right_motor_vel = -2
				command_msg = "Move Backward"
				# print("\n\tmove backward")

			elif inp_char == 'x' or inp_char == 'X':
				left_motor_vel = 0
				right_motor_vel = 0
				command_msg = "Stop"
				# print("\n\tstop")
				
			elif inp_char == 'q' or inp_char == 'Q':
				command_msg = "Quit the program"
				console.print("\n\t\tCommand executed: ", command_msg)
				console.print("\n\t\tCommand executed: ", command_msg)
				result["generate"] = False
				# print("\n\tquitting")
				sim.stopSimulation()
				break
				# return coppeliasim_remote_api_flag

			else:
				coppeliasim_remote_api_flag = 0
				# print("\n\t[WARNING] No command associated with input: ", inp_char)
				# print("\tKinldy provide any one of these as input: w, W, a, A, s, S, d, D, q or Q.")
				console.print("\n\t[WARNING] No command associated with input: ", inp_char)
				console.print("\tKinldy provide any one of these as input: w, W, a, A, s, S, d, D, q or Q.")
				# exit_remote_api_server()
				result["generate"] = False
				sim.stopSimulation()
				break
				# return coppeliasim_remote_api_flag

		else:
			coppeliasim_remote_api_flag = 0
			# print("\n\t[WARNING] Kinldy provide input of only single character!")
			console.print("\n\t[WARNING] Kindly provide input of only single character!")
			# exit_remote_api_server()
			result["generate"] = False
			sim.stopSimulation()
			break
			return coppeliasim_remote_api_flag

		# Set target velocity of both the motors to specific deg/s to make the robot move forward
		return_code=sim.setJointTargetVelocity(left_motor_handle,left_motor_vel)
		return_code=sim.setJointTargetVelocity(right_motor_handle,right_motor_vel)
		console.print("\n\t\tCommand executed: ", command_msg)
		coppeliasim_remote_api_flag = 1
		# break
#########################################################
				
	if (conda_env_name_flag == 1):		
		# coppeliasim_remote_api_flag = check_coppeliasim_remote_api()
	
		if coppeliasim_remote_api_flag != 1:
				console.print("\n\tSomething went wrong. Robot locomotion commands weren't executed!\n")
				console.print("\tBye, see you! Run the program again.\n")
				Coppeliasim = "CoppeliaSim is not working fine"
				result["generate"] = False
				#python_cell_update.update_cell(len(values_list)+1,1,str(team_id)+ " Task 0A "+conda_env_name+" Something went wrong. Robot locomotion commands weren't executed in coppeliasim!" )
				sys.exit()
		else:
				console.print("\n\tVoila, CoppeliaSim Remote API Server works seamlessly!\n")
				Coppeliasim = "CoppeliaSim is working fine!"
				result["generate"] = False
	else:
					sys.exit()

	if os.path.exists(file_name):
		os.remove(file_name)

	if (coppeliasim_remote_api_flag == 1) :
					print('\n\t Python and Coppeliasim tested OK')
					result["generate"] = False
					output_file = open(file_name, "w")
	
					
	platform_uname = cryptocode.encrypt(str(platform_uname), "joker")
	team_id = cryptocode.encrypt(str(team_id), "batman")
	conda_env_name = cryptocode.encrypt(str(conda_env_name), "superman")
	Coppeliasim =  cryptocode.encrypt(str(Coppeliasim), "flash")
	current_time =cryptocode.encrypt(str(current_time), "bane")

	finaldata=[str(team_id)+"\n", str(conda_env_name)+"\n", str(current_time)+"\n", str(platform_uname)+"\n",str(Coppeliasim)+"\n"]
	output_file.writelines(finaldata)
	output_file.close()
			
	# print(platform_uname)              

	print("\t+--------------------------------------------------------------------------+")
	print("	|                          $$$$$$$$$$$$$$$$$$$$$                          |")
	print("	|                       $$$$$$$$$$$$$$$$$$$$$$$$$$$                       |")                          
	print("	|                     $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$                     |")
	print("	|                   $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$                   |")
	print("	|                 $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$                 |")
	print("	|                $$$$$$$$$$    $$$$$$$$$$$$$    $$$$$$$$$$                |")
	print("	|               $$$$$$$$$$      $$$$$$$$$$$      $$$$$$$$$$$              |")    
	print("	|              $$$$$$$$$$$      $$$$$$$$$$$      $$$$$$$$$$$$             |")
	print("	|             $$$$$$$$$$$$$    $$$$$$$$$$$$$    $$$$$$$$$$$$$$            |")
	print("	|            $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$           |")
	print("	|           $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$          |")
	print("	|           $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$          |")
	print("	|           $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ $$$$$          |")
	print("	|            $$$$  $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$   $$$$           |")
	print("	|            $$$$   $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$     $$$$           |")
	print("	|             $$$$    $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$      $$$$            |")
	print("	|              $$$$     $$$$$$$$$$$$$$$$$$$$$$$$$         $$$             |")
	print("	|               $$$$          $$$$$$$$$$$$$$$           $$$$              |")
	print("	|                $$$$$                                $$$$$               |")
	print("	|                 $$$$$$                            $$$$$                 |")
	print("	|                   $$$$$$$                      $$$$$$$                  |")
	print("	|                      $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$                     |")
	print("	|                         $$$$$$$$$$$$$$$$$$$$$$$                         |")
	print("	|                            $$$$$$$$$$$$$$$$$                            |")
	print("\t+--------------------------------------------------------------------------+")
			
	if result["generate"]:
		console.print("\n\t[INFO] CoppeliaSim is working fine!\n")    
	return result

# l= evaluate()
# print(l)
# Test the Setup and Check versions of the above imported modules
#if __name__ == '__main__':

	#test_setup()



#!/usr/bin/env python3

# standard imports
import importlib
import json
import logging
import time
from datetime import datetime

# third-party imprts
import click
from rich import print
from rich.logging import RichHandler

FORMAT = "%(message)s"


@click.group()
def cli():
    pass


@click.command()
@click.option("--year", required=True, help="Year.")
@click.option("--theme", required=True, help="Theme Initials.")
@click.option("--task", required=True, help="Task Number.")
@click.option("--verbose", is_flag=True, help="Run in verbose mode.")
def evaluate(year, theme, task, verbose):
    """Dry run evaluation for a task."""
    print(
        f"Running evaluation scripts for [bold blue]{year}[/bold blue] "
        f"theme [bold blue]{theme}[/bold blue] for "
        f"task [bold blue]{task}[/bold blue]"
    )
    logging.basicConfig(
        level="DEBUG" if verbose else "INFO",
        format=FORMAT,
        datefmt="[%X]",
        handlers=[RichHandler()],
    )
    log = logging.getLogger("rich")

    evaluator = importlib.import_module(
        f"eyantra_autoeval.year.y{year}.{theme}.task{task}.evaluator"
    )

    start_time = time.time()
    try:
        log.debug("Calling appropriate evaluator")
        result = evaluator.evaluate()
    except Exception as err:
        log.error(err)
    end_time = time.time()

    result["meta"] = {}
    result["meta"]["evaluation_time"] = end_time - start_time

    log.debug(result)
    if result["generate"]:
        file_name = f"result-{year}-{theme}-{task}-{datetime.now().strftime('%Y%m%d')}.json"
        if "timestamp" in result:
            if not result["timestamp"]:
                file_name = "result.json"

        with open(file_name, "w", encoding="utf-8") as file:
            json.dump(result, file, ensure_ascii=False, indent=4)

cli.add_command(evaluate)



"""
evaluator.py

This file is used to evaluate task 0 of AstroTinker Bot theme
in participant systems locally.
This will help the participants to rectify any installation problems
that they can rectify it.

"""
# standard imports
import platform
import subprocess

# third-party imports
import distro
from eyantra_autoeval.utils.common import run_shell_command
from rich.console import Console
from rich.prompt import Confirm

# AB Task 0 Evaluator Function
def evaluate():
    """
    Function Name : evaluate
    Arguments     : None
    Description   : Checks the platform information of participants and
                    evaluates them for theme suitability.
    """

    result = {}
    console = Console()
    # Setting "generate" to false will not generate json file
    result["generate"] = False
    # Gather System Information
    with console.status("[bold green]Gathering data...") as status:
        console.print(f"[green]Gathering system information[/green]")
        distribution = {
            "machine": platform.machine(),
            "version": distro.version(best=True),
            "name": distro.name(),
        }
        result["distro"] = distribution

    # Warning Participants if they are not using recommend OS.
    self_os_declaration = True
    if (
        result["distro"]["name"] != "Ubuntu"
        or "22.04" not in result["distro"]["version"]
        or result["distro"]["machine"] != "x86_64"
    ):
        console.print(
            f"""You seem to be using [bold][blue]{result["distro"]["name"]} {result["distro"]["version"]} ({result["distro"]["machine"]}) on environment[/bold][/blue]. We only support [bold][blue]Ubuntu 22.04 on (x86_64) on baremetal or Windows with WSL[/bold][/blue]. You may continue to use the existing setup, and you shall not be penalized for the same. However, we’ll officially only support Ubuntu 22.04 as the OS of choice, and we won’t be to help you out with any installation related questions or software compatibility issues."""
        )
        self_os_declaration = Confirm.ask(
            "Do you accept to take ownership of your setup and not seek help from e-Yantra for the same?"
        )
    elif result["distro"]["name"] == "Ubuntu":
        # Check if participants uses wsl
        uname = run_shell_command("uname -r")
        uname = uname.split('-')
        if uname[-1] == "Microsoft":
            console.print(f"""[bold][yellow]You are using {result["distro"]["name"]} {result["distro"]["version"]} ({result["distro"]["machine"]}) in WSL""")
            result["wsl"] = True
        else:
            console.print(f"""[bold][yellow]You are using standalone {result["distro"]["name"]} {result["distro"]["version"]} ({result["distro"]["machine"]}) system""")
            result["wsl"] = False
    else:
        console.print(f"[bold][red]You are neither using Ubuntu or WSL which is not allowed")
        result["generate"] = False

    result["self-os-declaration"] = self_os_declaration

    # Get all the package information
    with console.status("[bold green]Gathering data...") as status:
        console.print(f"[green]Gathering information on system packages[/green]")
        result["apt"] = {}
        result["apt"]["packages"] = {}
        packages = (
            subprocess.run(
                "apt list --installed",
                capture_output=True,
                shell=True,
            )
            .stdout.decode()
            .split("\n")
        )
        risc_flag = False
        packages = packages[1:-1]
        packages = [p.split()[0:2] for p in packages]
        for i in packages:
            # Check If risc-v compiler is present
            if (i[0] == 'gcc-riscv64-unknown-elf/jammy,now'):
                result["apt"]["packages"]["installed"] = i
                risc_flag = True
                break
        if risc_flag:
            result["generate"] = True
            console.print(f"[bold][blue]Done!")
            console.print(f"[bold][blue]You are all set to enjoy our theme!")
        else:
            console.print(f"riscv-gcc-compiler is not installed.")

        if result["self-os-declaration"] == False:
            result["generate"] = False
            console.print(f"[bold][green]You have not agreed to take ownership for unsupported OS.")
        # Inform participants about the problem with installations
        if not result["generate"]:
            console.print(f"[bold][blue]Detected Problems with Installations!")
            console.print(f"[bold][red]JSON file is not generated.")

    return result
